@@ -1,31 +1,33 @@
 /*
- * Copyright 2002, 2003 Niels Provos <provos@citi.umich.edu>
+ * Copyright (c) 2002, 2003, 2004 Niels Provos <provos@citi.umich.edu>
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Niels Provos.
- * 4. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include <sys/param.h>
@@ -143,6 +145,10 @@ struct tcp_con		 honeyd_tmp;
 int                      honeyd_show_include_dir;
 int                      honeyd_show_version;
 int                      honeyd_show_usage;
+int			 honeyd_debug;
+uid_t			 honeyd_uid = 32767;
+gid_t			 honeyd_gid = 32767;
+int			 honeyd_needsroot;	/* Need different IDs */
 
 static struct option honeyd_long_opts[] = {
 	{"include-dir", 0, &honeyd_show_include_dir, 1},
@@ -166,6 +172,8 @@ usage(void)
 		"  -x file                Read xprobe-style fingerprints from file.\n"
 		"  -a assocfile           Read nmap-xprobe associations from file.\n"
 		"  -0 osfingerprints      Read pf-style OS fingerprints from file.\n"
+		"  -u uid		  Set the uid Honeyd should run as.\n"
+		"  -g gid		  Set the gid Honeyd should run as.\n"
 		"  -f configfile          Read configuration from file.\n"
 		"  -V, --version          Print program version and exit.\n"
 		"  -h, --help             Print this message and exit.\n"
@@ -215,7 +223,7 @@ honeyd_setudp(struct udp_con *con, struct ip_hdr *ip, struct udp_hdr *udp,
 }
 
 char *
-honeyd_contoa(struct tuple *hdr)
+honeyd_contoa(const struct tuple *hdr)
 {
 	static char buf[128];
 	char asrc[24], adst[24];
@@ -482,7 +490,7 @@ honeyd_delay_cb(int fd, short which, void *arg)
 
 void
 honeyd_delay_packet(struct template *tmpl, struct ip_hdr *ip, u_int iplen,
-    struct addr *src, struct addr *dst, int ms, int flags)
+    const struct addr *src, const struct addr *dst, int ms, int flags)
 {
 	struct delay *delay, tmp_delay;
 	struct timeval tv;
@@ -512,7 +520,7 @@ honeyd_delay_packet(struct template *tmpl, struct ip_hdr *ip, u_int iplen,
 		memset(&tmp_delay, 0, sizeof(tmp_delay));
 		delay = &tmp_delay;
 	}
-	delay->ip = ip;
+ 	delay->ip = ip;
 	delay->iplen = iplen;
 
 	if (src != NULL)
@@ -1029,7 +1037,23 @@ tcp_send(struct tcp_con *con, uint8_t flags, u_char *payload, u_int len)
 	if (con->window)
 		window = con->window;
 
-	options = tcp_personality(con, &flags, &window, &dontfragment, &id);
+	/*
+	 * The TCP personality will always set snd_una for us if necessary.
+	 * snd_una maybe 0 on RST segments.
+	 */
+	if (tcp_personality(con, &flags, &window, &dontfragment, &id,
+		&options) == -1) {
+		/* 
+		 * If we do not match a personality and sent a reset
+		 * segment then we do not want to include options.
+		 */
+		if (flags & TH_RST) {
+			options = NULL;
+			window = con->window;
+		} else if (flags & TH_SYN) {
+			options = "m";
+		}
+	}
 
 	/* Empty flags indicates packet drop */
 	if (flags == 0)
@@ -1048,10 +1072,6 @@ tcp_send(struct tcp_con *con, uint8_t flags, u_char *payload, u_int len)
 			window = 0;
 	}
 
-	/* Set sequence number */
-	if (con->snd_una == 0)
-		con->snd_una = rand_uint32(honeyd_rand);
-
 	pkt = pool_alloc(pool_pkt);
 
 	tcp = (struct tcp_hdr *)(pkt + IP_HDR_LEN);
@@ -1532,6 +1552,7 @@ generic_timeout(struct event *ev, int seconds)
 void
 tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 {
+	char *comment = NULL;
 	struct ip_hdr *ip;
 	struct tcp_hdr *tcp;
 	struct tcp_con *con;
@@ -1541,7 +1562,7 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 	uint16_t th_sum;
 	u_char *data;
 	u_int dlen, doff;
-	uint8_t tiflags;
+	uint8_t tiflags, flags;
 
 	ip = (struct ip_hdr *)pkt;
 	tcp = (struct tcp_hdr *)(pkt + (ip->ip_hl << 2));
@@ -1564,21 +1585,43 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 	honeyd_tmp.state = action == NULL || PORT_ISOPEN(action) ? 
 	    TCP_STATE_LISTEN : TCP_STATE_CLOSED;
 
+	tiflags = tcp->th_flags;
+
 	con = (struct tcp_con *)SPLAY_FIND(tree, &tcpcons, &honeyd_tmp.conhdr);
 	if (con == NULL) {
 		if (honeyd_tmp.state != TCP_STATE_LISTEN)
 			goto kill;
-
-		if ((tcp->th_flags & TH_SYN) == 0) 
+		if ((tiflags & TH_SYN) == 0) 
 			goto kill;
 
-		if (tcp->th_flags & ~TH_SYN &
+		if (tiflags & ~TH_SYN &
 		    (TH_FIN|TH_RST|TH_PUSH|TH_ACK|TH_URG)) {
-			int win = 16000, df = 0;
-			uint8_t flags = TH_SYN|TH_ACK;
+			int win = 0, df = 0;
 			uint16_t id;
-			/* Check what the flags say */
-			tcp_personality(&honeyd_tmp, &flags, &win, &df, &id);
+
+			flags = TH_SYN|TH_ACK;
+			if (tiflags & (TH_FIN|TH_RST))
+				comment = " {Honeyd Scanner?}";
+
+			/* 
+			 * Some stacks might reply to a packet like
+			 * this.  So, check the personalities and see
+			 * what the flags say.
+			 */
+			if (tcp_personality(&honeyd_tmp,
+				&flags, &win, &df, &id, NULL) == -1) {
+				/* 
+				 * These flags normally cause a termination,
+				 * so drop or reset the connection as we did
+				 * not match a fingerprint.
+				 */
+				if (tiflags & (TH_RST|TH_ACK))
+					goto kill;
+			}
+
+			tiflags &= ~TH_FIN;
+
+			/* Just drop the packet */
 			if (flags & TH_RST)
 				goto kill;
 		}
@@ -1598,6 +1641,7 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 		if ((con = tcp_new(ip, tcp, 0)) == NULL) {
 			goto kill;
 		}
+		con->rcv_flags = tiflags;
 
 		/* Check if this connection is a tar pit */
 		if (action != NULL && (action->flags & PORT_TARPIT))
@@ -1627,7 +1671,7 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 	th_seq = ntohl(tcp->th_seq);
 	th_ack = ntohl(tcp->th_ack);
 
-	con->rcv_flags = tiflags = tcp->th_flags;
+	con->rcv_flags = tiflags;
 
 	switch (con->state) {
 	case TCP_STATE_SYN_SENT:
@@ -1748,10 +1792,10 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 
  kill:
 	honeyd_log_probe(honeyd_logfp, IP_PROTO_TCP, &honeyd_tmp.conhdr,
-	    pktlen, tcp->th_flags);
+	    pktlen, tcp->th_flags, comment);
 
 	/* Do not kill on reset */
-	if (tcp->th_flags & TH_RST)
+	if (tiflags & TH_RST)
 		return;
 
 	syslog(LOG_DEBUG, "Killing %s connection: tcp %s",
@@ -1759,16 +1803,24 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 	    honeyd_contoa(&honeyd_tmp.conhdr));
 
 	/* Fake connection element */
-	honeyd_tmp.rcv_next = ntohl(tcp->th_seq) + 1;
-	honeyd_tmp.snd_una = ntohl(tcp->th_ack);
+	if (tiflags & TH_ACK) {
+		flags = TH_RST;
+		honeyd_tmp.rcv_next = 0;
+		honeyd_tmp.snd_una = ntohl(tcp->th_ack);
+	} else {
+		flags = TH_RST | TH_ACK;
+		honeyd_tmp.rcv_next = ntohl(tcp->th_seq) + 1;
+		honeyd_tmp.snd_una = 0;
+	}
 	honeyd_tmp.tmpl = tmpl;
 
-	/* Even though options processing does not make any sense on 
+	/* 
+	 * Even though options processing does not make any sense on 
 	 * RST segment, some stacks apparently do it anyway.
 	 */
 	tcp_do_options(&honeyd_tmp, tcp);
 
-	tcp_send(&honeyd_tmp, TH_RST|TH_ACK, NULL, 0);
+	tcp_send(&honeyd_tmp, flags, NULL, 0);
 	return;
 
  close:
@@ -1796,7 +1848,7 @@ tcp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
  justlog:
 	honeyd_settcp(&honeyd_tmp, ip, tcp, 0);
 	honeyd_log_probe(honeyd_logfp, IP_PROTO_TCP,&honeyd_tmp.conhdr,
-	    pktlen, tcp->th_flags);
+	    pktlen, tcp->th_flags, comment);
 }
 
 int
@@ -1910,7 +1962,7 @@ udp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 
  closed:
 	honeyd_log_probe(honeyd_logfp, IP_PROTO_UDP, &honeyd_udp.conhdr,
-	    pktlen, 0);
+	    pktlen, 0, NULL);
 
 	addr_pack(&addr, ADDR_TYPE_IP, IP_ADDR_BITS, &ip->ip_dst, IP_ADDR_LEN);
 	icmp_error_send(tmpl, &addr, ICMP_UNREACH, ICMP_UNREACH_PORT, ip); 
@@ -1919,7 +1971,7 @@ udp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
  justlog:
 	honeyd_setudp(&honeyd_udp, ip, udp, 0);
 	honeyd_log_probe(honeyd_logfp, IP_PROTO_UDP, &honeyd_udp.conhdr,
-	    pktlen, 0);
+	    pktlen, 0, NULL);
 }
 
 void
@@ -1956,7 +2008,7 @@ icmp_recv_cb(struct template *tmpl, u_char *pkt, u_short pktlen)
 	icmphdr.type = SOCK_RAW;
 	icmphdr.sport = icmp->icmp_type; /* XXX - horrible cludge */
 	icmphdr.dport = icmp->icmp_code;
-	honeyd_log_probe(honeyd_logfp, IP_PROTO_ICMP, &icmphdr, pktlen, 0);
+	honeyd_log_probe(honeyd_logfp, IP_PROTO_ICMP, &icmphdr, pktlen, 0, NULL);
 
 	/* We can block ICMP, too */
 	if (tmpl && tmpl->icmp.status == PORT_BLOCK)
@@ -2113,7 +2165,7 @@ honeyd_dispatch(struct template *tmpl, struct ip_hdr *ip, u_short iplen)
 		iphdr.ip_src = ip->ip_src;
 		iphdr.ip_dst = ip->ip_dst;
 		iphdr.type = -1;
-		honeyd_log_probe(honeyd_logfp, ip->ip_p, &iphdr, iplen, 0);
+		honeyd_log_probe(honeyd_logfp, ip->ip_p, &iphdr, iplen, 0, NULL);
 		return;
 	}
 }
@@ -2292,7 +2344,7 @@ honeyd_route_packet(struct ip_hdr *ip, u_int iplen,
 }
 
 void
-honeyd_input(struct interface *inter, struct ip_hdr *ip, u_short iplen)
+honeyd_input(const struct interface *inter, struct ip_hdr *ip, u_short iplen)
 {
 	extern struct network *reverse;
 	struct template *tmpl = NULL;
@@ -2398,7 +2450,7 @@ honeyd_input(struct interface *inter, struct ip_hdr *ip, u_short iplen)
 void
 honeyd_recv_cb(u_char *ag, const struct pcap_pkthdr *pkthdr, const u_char *pkt)
 {
-	struct interface *inter = (struct interface *)ag;
+	const struct interface *inter = (const struct interface *)ag;
 	struct ip_hdr *ip;
 	struct addr addr;
 	u_short iplen;
@@ -2476,16 +2528,36 @@ main(int argc, char *argv[])
 	char *osfp = PATH_HONEYDDATA "/pf.os";
 	char *logfile = NULL;
 	char *servicelog = NULL;
-	int i, c, debug, orig_argc, ninterfaces = 0;
+	int setrand = 0;
+	int i, c, orig_argc, ninterfaces = 0;
 	FILE *fp;
 
-	debug = 0;
-	
 	orig_argc = argc;
 	orig_argv = argv;
-	while ((c = getopt_long(argc, argv, "VPdi:p:x:a:f:l:s:0:h?",
+	while ((c = getopt_long(argc, argv, "VPdi:p:x:a:u:g:f:l:s:0:R:h?",
 				honeyd_long_opts, NULL)) != -1) {
+		char *ep;
 		switch (c) {
+		case 'R':
+			/* For regression testing */
+			setrand = atoi(optarg);
+			break;
+		case 'u':
+			honeyd_uid = strtoul(optarg, &ep, 10);
+			honeyd_needsroot = -1;
+			if (optarg[0] == '\0' || *ep != '\0') {
+				fprintf(stderr, "Bad uid %s\n", optarg);
+				usage();
+			}
+			break;
+		case 'g':
+			honeyd_gid = strtoul(optarg, &ep, 10);
+			honeyd_needsroot = -1;
+			if (optarg[0] == '\0' || *ep != '\0') {
+				fprintf(stderr, "Bad gid %s\n", optarg);
+				usage();
+			}
+			break;
 		case 'V':
 			honeyd_show_version = 1;
 			break;
@@ -2493,7 +2565,7 @@ main(int argc, char *argv[])
 			honeyd_dopoll = 1;
 			break;
 		case 'd':
-			debug = 1;
+			honeyd_debug++;
 			break;
 		case 'i':
 			if (ninterfaces >= HONEYD_MAX_INTERFACES)
@@ -2548,6 +2620,9 @@ main(int argc, char *argv[])
 
 	if ((honeyd_rand = rand_open()) == NULL)
 		err(1, "rand_open");
+	/* We need reproduceable random numbers for regression testing */
+	if (setrand)
+		rand_set(honeyd_rand, &setrand, sizeof(setrand));
 
 	/* Initalize libevent but without kqueue because of bpf */
 	setenv("EVENT_NOKQUEUE", "yes", 0);
@@ -2625,10 +2700,13 @@ main(int argc, char *argv[])
 	
 	ip_fragment_init();
 
+	/* Create PID file, we might not be able to remove it */
+	unlink(PIDFILE);
 	if ((fp = fopen(PIDFILE, "w")) == NULL)
 		err(1, "fopen");
 
-	if (!debug) {
+	/* Start Honeyd in the background if necessary */
+	if (!honeyd_debug) {
 		setlogmask(LOG_UPTO(LOG_INFO));
 		
 		fprintf(stderr, "Honeyd starting as background process\n");
@@ -2637,11 +2715,41 @@ main(int argc, char *argv[])
 			err(1, "daemon");
 		}
 	}
+	
 	fprintf(fp, "%d\n", getpid());
 	fclose(fp);
 	
 	chmod(PIDFILE, 0644);
 
+	/* Drop privileges if we do not need them */
+	if (honeyd_needsroot <= 0) {
+#ifdef HAVE_SETGROUPS
+		if (setgroups(1, &honeyd_gid) == -1)
+			err(1, "setgroups");
+#endif
+#ifdef HAVE_SETREGID
+		if (setregid(honeyd_gid, honeyd_gid) == -1)
+			err(1, "setregid");
+#endif
+		if (setegid(honeyd_gid) == -1)
+			err(1, "setegid");
+		if (setgid(honeyd_gid) == -1)
+			err(1, "setgid");
+#ifdef HAVE_SETREUID
+		if (setreuid(honeyd_uid, honeyd_uid) == -1)
+			err(1, "setreuid");
+#endif
+#ifdef __OpenBSD__
+		if (seteuid(honeyd_uid) == -1)
+			err(1, "seteuid");
+#endif
+		if (setuid(honeyd_uid) == -1)
+			err(1, "setuid");
+		syslog(LOG_NOTICE,
+		    "Demoting process privileges to uid %u, gid %u",
+		    honeyd_uid, honeyd_gid);
+	}
+
 	/* Setup signal handler */
 	if (signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
 		perror("signal");
@@ -2668,3 +2776,22 @@ main(int argc, char *argv[])
 
 	return (0);
 }
+
+/*
+ * Determine if Honeyd should automatically demote the user id it is
+ * going to use.
+ */
+
+void
+honeyd_use_uid(uid_t uid)
+{
+	if (!honeyd_needsroot && uid != honeyd_uid)
+		honeyd_needsroot = 1;
+}
+
+void
+honeyd_use_gid(gid_t gid)
+{
+	if (!honeyd_needsroot && gid != honeyd_gid)
+		honeyd_needsroot = 1;
+}
