@@ -68,7 +68,7 @@
 # define	PROTO	319
 # define	FLOAT	320
 
-#line 30 "parse.y"
+#line 5 "parse.y"
 
 #include <sys/types.h>
 
@@ -127,7 +127,7 @@ int errors = 0;
 int curtype = -1;	/* Lex sets it to SOCK_STREAM or _DGRAM */
 
 
-#line 119 "parse.y"
+#line 94 "parse.y"
 #ifndef YYSTYPE
 typedef union {
 	char *string;
@@ -254,15 +254,15 @@ static const short yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined. */
 static const short yyrline[] =
 {
-       0,   136,   137,   138,   139,   140,   141,   142,   143,   144,
-     147,   153,   158,   168,   176,   184,   193,   208,   215,   238,
-     266,   274,   292,   298,   304,   315,   326,   336,   348,   354,
-     361,   367,   373,   402,   413,   424,   437,   438,   440,   441,
-     442,   444,   450,   470,   491,   503,   510,   520,   534,   542,
-     568,   584,   590,   596,   605,   612,   618,   625,   633,   640,
-     644,   649,   650,   655,   656,   661,   662,   666,   671,   672,
-     681,   692,   703,   715,   731,   735,   741,   745,   751,   764,
-     772,   780,   790,   797
+       0,   111,   112,   113,   114,   115,   116,   117,   118,   119,
+     122,   128,   133,   143,   151,   159,   168,   183,   190,   213,
+     241,   249,   267,   273,   279,   290,   301,   312,   326,   332,
+     339,   345,   351,   380,   391,   402,   415,   416,   418,   419,
+     420,   422,   428,   448,   469,   481,   488,   498,   512,   520,
+     546,   562,   568,   574,   583,   590,   596,   603,   611,   618,
+     622,   627,   628,   633,   634,   639,   640,   644,   649,   650,
+     659,   670,   681,   693,   709,   713,   719,   723,   729,   742,
+     750,   758,   768,   775
 };
 #endif
 
@@ -1127,7 +1127,7 @@ yyreduce:
   switch (yyn) {
 
 case 10:
-#line 148 "parse.y"
+#line 123 "parse.y"
 {
 		if (template_create(yyvsp[0].string) == NULL)
 			yyerror("Template \"%s\" exists already", yyvsp[0].string);
@@ -1135,14 +1135,14 @@ case 10:
 	}
     break;
 case 11:
-#line 154 "parse.y"
+#line 129 "parse.y"
 {
 		if (template_create("default") == NULL)
 			yyerror("Template \"default\" exists already");
 	}
     break;
 case 12:
-#line 159 "parse.y"
+#line 134 "parse.y"
 {		
 		struct template *tmpl;
 		if ((tmpl = template_create(yyvsp[0].string)) == NULL)
@@ -1152,7 +1152,7 @@ case 12:
 	}
     break;
 case 13:
-#line 169 "parse.y"
+#line 144 "parse.y"
 {
 		if (yyvsp[-4].tmpl != NULL && template_add(yyvsp[-4].tmpl, yyvsp[-3].number, yyvsp[-1].number, &yyvsp[0].action) == -1)
 			yyerror("Can not add port %d to template \"%s\"",
@@ -1162,7 +1162,7 @@ case 13:
 	}
     break;
 case 14:
-#line 177 "parse.y"
+#line 152 "parse.y"
 {	
 		if (yyvsp[-4].tmpl == NULL || yyvsp[-2].tmpl == NULL)
 			break;
@@ -1172,7 +1172,7 @@ case 14:
 	}
     break;
 case 15:
-#line 185 "parse.y"
+#line 160 "parse.y"
 {	
 		if (yyvsp[-3].tmpl == NULL || yyvsp[0].tmpl == NULL)
 			break;
@@ -1182,7 +1182,7 @@ case 15:
 	}
     break;
 case 16:
-#line 194 "parse.y"
+#line 169 "parse.y"
 {
 		struct addr tmp;
 		int isaddr;
@@ -1198,7 +1198,7 @@ case 16:
 	}
     break;
 case 17:
-#line 209 "parse.y"
+#line 184 "parse.y"
 {
 		/* Bind to an IP address and start subsystems */
 		if (yyvsp[0].tmpl == NULL ||
@@ -1207,7 +1207,7 @@ case 17:
 	}
     break;
 case 18:
-#line 216 "parse.y"
+#line 191 "parse.y"
 {
 		struct template *tmpl;
 
@@ -1232,7 +1232,7 @@ case 18:
 	}
     break;
 case 19:
-#line 239 "parse.y"
+#line 214 "parse.y"
 {
 		struct interface *inter;
 		struct template *tmpl;
@@ -1262,7 +1262,7 @@ case 19:
 	}
     break;
 case 20:
-#line 267 "parse.y"
+#line 242 "parse.y"
 {
 		/* Just clone.  This is not the final destination yet */
 		if (yyvsp[0].tmpl == NULL || template_clone(yyvsp[-1].string, yyvsp[0].tmpl, 0) == NULL)
@@ -1271,7 +1271,7 @@ case 20:
 	}
     break;
 case 21:
-#line 275 "parse.y"
+#line 250 "parse.y"
 {
 		struct action *action;
 
@@ -1291,7 +1291,7 @@ case 21:
 	}
     break;
 case 22:
-#line 293 "parse.y"
+#line 268 "parse.y"
 {
 		if (yyvsp[-2].tmpl == NULL || yyvsp[0].pers == NULL)
 			break;
@@ -1299,7 +1299,7 @@ case 22:
 	}
     break;
 case 23:
-#line 299 "parse.y"
+#line 274 "parse.y"
 {
 		if (yyvsp[-2].tmpl == NULL || yyvsp[0].number == 0)
 			break;
@@ -1307,7 +1307,7 @@ case 23:
 	}
     break;
 case 24:
-#line 305 "parse.y"
+#line 280 "parse.y"
 {
 		if (yyvsp[-3].tmpl == NULL)
 			break;
@@ -1320,7 +1320,7 @@ case 24:
 	}
     break;
 case 25:
-#line 316 "parse.y"
+#line 291 "parse.y"
 {
 		if (yyvsp[-3].tmpl == NULL)
 			break;
@@ -1333,7 +1333,7 @@ case 25:
 	}
     break;
 case 26:
-#line 327 "parse.y"
+#line 302 "parse.y"
 {
 		if (yyvsp[-2].tmpl == NULL)
 			break;
@@ -1342,10 +1342,11 @@ case 26:
 			break;
 		}
 		yyvsp[-2].tmpl->uid = yyvsp[0].number;
+		honeyd_use_uid(yyvsp[0].number);
 	}
     break;
 case 27:
-#line 337 "parse.y"
+#line 313 "parse.y"
 {
 		if (yyvsp[-4].tmpl == NULL)
 			break;
@@ -1355,10 +1356,12 @@ case 27:
 		}
 		yyvsp[-4].tmpl->uid = yyvsp[-2].number;
 		yyvsp[-4].tmpl->gid = yyvsp[0].number;
+		honeyd_use_uid(yyvsp[-2].number);
+		honeyd_use_gid(yyvsp[0].number);
 	}
     break;
 case 28:
-#line 349 "parse.y"
+#line 327 "parse.y"
 {
 		if (yyvsp[-1].pers == NULL)
 			break;
@@ -1366,7 +1369,7 @@ case 28:
 	}
     break;
 case 29:
-#line 355 "parse.y"
+#line 333 "parse.y"
 {
 		if (yyvsp[-1].pers == NULL)
 			break;
@@ -1374,7 +1377,7 @@ case 29:
 	}
     break;
 case 30:
-#line 362 "parse.y"
+#line 340 "parse.y"
 {
 		if (router_start(&yyvsp[0].addr, NULL) == -1)
 			yyerror("Defining entry point failed: %s",
@@ -1382,7 +1385,7 @@ case 30:
 	}
     break;
 case 31:
-#line 368 "parse.y"
+#line 346 "parse.y"
 {
 		if (router_start(&yyvsp[-2].addr, &yyvsp[0].addr) == -1)
 			yyerror("Defining entry point failed: %s",
@@ -1390,7 +1393,7 @@ case 31:
 	}
     break;
 case 32:
-#line 374 "parse.y"
+#line 352 "parse.y"
 {
 		struct router *r, *newr;
 		struct addr defroute;
@@ -1421,7 +1424,7 @@ case 32:
 	}
     break;
 case 33:
-#line 403 "parse.y"
+#line 381 "parse.y"
 {
 		struct router *r;
 
@@ -1434,7 +1437,7 @@ case 33:
 	}
     break;
 case 34:
-#line 414 "parse.y"
+#line 392 "parse.y"
 {
 		struct router *r;
 
@@ -1447,7 +1450,7 @@ case 34:
 	}
     break;
 case 35:
-#line 425 "parse.y"
+#line 403 "parse.y"
 {
 		struct router *r;
 
@@ -1461,27 +1464,27 @@ case 35:
 	}
     break;
 case 36:
-#line 437 "parse.y"
+#line 415 "parse.y"
 { yyval.number = 1; }
     break;
 case 37:
-#line 438 "parse.y"
+#line 416 "parse.y"
 { yyval.number = 0; }
     break;
 case 38:
-#line 440 "parse.y"
+#line 418 "parse.y"
 { yyval.fragp = FRAG_DROP; }
     break;
 case 39:
-#line 441 "parse.y"
+#line 419 "parse.y"
 { yyval.fragp = FRAG_OLD; }
     break;
 case 40:
-#line 442 "parse.y"
+#line 420 "parse.y"
 { yyval.fragp = FRAG_NEW; }
     break;
 case 41:
-#line 445 "parse.y"
+#line 423 "parse.y"
 {
 		if (addr_pton(yyvsp[0].string, &yyval.addr) < 0)
 			yyerror("Illegal IP address %s", yyvsp[0].string);
@@ -1489,7 +1492,7 @@ case 41:
 	}
     break;
 case 42:
-#line 451 "parse.y"
+#line 429 "parse.y"
 {
 		struct addrinfo ai, *aitop;
 
@@ -1510,7 +1513,7 @@ case 42:
 	}
     break;
 case 43:
-#line 471 "parse.y"
+#line 449 "parse.y"
 {
 		char src[25];
 		struct addr b;
@@ -1532,7 +1535,7 @@ case 43:
 	}
     break;
 case 44:
-#line 492 "parse.y"
+#line 470 "parse.y"
 {
 		if (curtype == -1) {
 			yyerror("Bad port type");
@@ -1545,7 +1548,7 @@ case 44:
 	}
     break;
 case 45:
-#line 504 "parse.y"
+#line 482 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyval.action.action = yyvsp[0].string;
@@ -1554,7 +1557,7 @@ case 45:
 	}
     break;
 case 46:
-#line 511 "parse.y"
+#line 489 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyvsp[0].string[strlen(yyvsp[0].string) - 1] = '\0';
@@ -1566,7 +1569,7 @@ case 46:
 	}
     break;
 case 47:
-#line 521 "parse.y"
+#line 499 "parse.y"
 {
 #ifdef HAVE_PYTHON
 		memset(&yyval.action, 0, sizeof(yyval.action));
@@ -1582,7 +1585,7 @@ case 47:
 	}
     break;
 case 48:
-#line 535 "parse.y"
+#line 513 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyval.action.status = PORT_PROXY;
@@ -1592,7 +1595,7 @@ case 48:
 	}
     break;
 case 49:
-#line 543 "parse.y"
+#line 521 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyval.action.status = PORT_PROXY;
@@ -1620,7 +1623,7 @@ case 49:
 	}
     break;
 case 50:
-#line 569 "parse.y"
+#line 547 "parse.y"
 {
 		char proxy[1024];
 		memset(&yyval.action, 0, sizeof(yyval.action));
@@ -1638,7 +1641,7 @@ case 50:
 	}
     break;
 case 51:
-#line 585 "parse.y"
+#line 563 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyval.action.status = PORT_BLOCK;
@@ -1646,7 +1649,7 @@ case 51:
 	}
     break;
 case 52:
-#line 591 "parse.y"
+#line 569 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyval.action.status = PORT_RESET;
@@ -1654,7 +1657,7 @@ case 52:
 	}
     break;
 case 53:
-#line 597 "parse.y"
+#line 575 "parse.y"
 {
 		memset(&yyval.action, 0, sizeof(yyval.action));
 		yyval.action.status = PORT_OPEN;
@@ -1663,7 +1666,7 @@ case 53:
 	}
     break;
 case 54:
-#line 606 "parse.y"
+#line 584 "parse.y"
 {
 		yyval.tmpl = template_find(yyvsp[0].string);
 		if (yyval.tmpl == NULL)
@@ -1672,7 +1675,7 @@ case 54:
 	}
     break;
 case 55:
-#line 613 "parse.y"
+#line 591 "parse.y"
 {
 		yyval.tmpl = template_find("default");
 		if (yyval.tmpl == NULL)
@@ -1680,7 +1683,7 @@ case 55:
 	}
     break;
 case 56:
-#line 619 "parse.y"
+#line 597 "parse.y"
 {
 		yyval.tmpl = template_find(addr_ntoa(&yyvsp[0].addr));
 		if (yyval.tmpl == NULL)
@@ -1688,7 +1691,7 @@ case 56:
 	}
     break;
 case 57:
-#line 626 "parse.y"
+#line 604 "parse.y"
 {
 		yyvsp[0].string[strlen(yyvsp[0].string) - 1] = '\0';
 		yyval.pers = personality_find(yyvsp[0].string+1);
@@ -1698,7 +1701,7 @@ case 57:
 	}
     break;
 case 58:
-#line 634 "parse.y"
+#line 612 "parse.y"
 {
 		yyval.pers = personality_random();
 		if (yyval.pers == NULL)
@@ -1706,59 +1709,59 @@ case 58:
 	}
     break;
 case 59:
-#line 641 "parse.y"
+#line 619 "parse.y"
 {
 		yyval.floatp = yyvsp[0].floatp;
 	}
     break;
 case 60:
-#line 645 "parse.y"
+#line 623 "parse.y"
 {
 		yyval.floatp = yyvsp[0].number;
 	}
     break;
 case 61:
-#line 649 "parse.y"
+#line 627 "parse.y"
 { yyval.number = 0; }
     break;
 case 62:
-#line 651 "parse.y"
+#line 629 "parse.y"
 {
 		yyval.number = yyvsp[-1].number;
 	}
     break;
 case 63:
-#line 655 "parse.y"
+#line 633 "parse.y"
 { yyval.number = 0; }
     break;
 case 64:
-#line 657 "parse.y"
+#line 635 "parse.y"
 {
 		yyval.number = yyvsp[0].floatp * 100;
 	}
     break;
 case 65:
-#line 661 "parse.y"
+#line 639 "parse.y"
 { yyval.number = 0; }
     break;
 case 66:
-#line 663 "parse.y"
+#line 641 "parse.y"
 {
 		yyval.number = yyvsp[-1].number * yyvsp[0].number;
 	}
     break;
 case 67:
-#line 667 "parse.y"
+#line 645 "parse.y"
 {
 		yyval.number = yyvsp[0].number;
 	}
     break;
 case 68:
-#line 671 "parse.y"
+#line 649 "parse.y"
 { memset(&yyval.drop, 0, sizeof(yyval.drop)); }
     break;
 case 69:
-#line 673 "parse.y"
+#line 651 "parse.y"
 {
 		if (yyvsp[-1].number <= yyvsp[-4].number)
 			yyerror("Incorrect thresholds. First number needs to "
@@ -1768,7 +1771,7 @@ case 69:
 	}
     break;
 case 70:
-#line 682 "parse.y"
+#line 660 "parse.y"
 {
 		struct honeyd_plugin_cfg cfg;
 
@@ -1781,7 +1784,7 @@ case 70:
 	}
     break;
 case 71:
-#line 693 "parse.y"
+#line 671 "parse.y"
 {
 		struct honeyd_plugin_cfg cfg;
 
@@ -1794,7 +1797,7 @@ case 71:
         }
     break;
 case 72:
-#line 704 "parse.y"
+#line 682 "parse.y"
 {
 		struct honeyd_plugin_cfg cfg;
 
@@ -1807,7 +1810,7 @@ case 72:
         }
     break;
 case 73:
-#line 716 "parse.y"
+#line 694 "parse.y"
 {
 		struct honeyd_plugin_cfg cfg;
 		char path[MAXPATHLEN];
@@ -1823,31 +1826,31 @@ case 73:
         }
     break;
 case 74:
-#line 732 "parse.y"
+#line 710 "parse.y"
 {
 	yyval.number = 0;
 }
     break;
 case 75:
-#line 736 "parse.y"
+#line 714 "parse.y"
 {
 	yyval.number = 1;
 }
     break;
 case 76:
-#line 742 "parse.y"
+#line 720 "parse.y"
 {
 	yyval.number = 0;
 }
     break;
 case 77:
-#line 746 "parse.y"
+#line 724 "parse.y"
 {
 	yyval.number = PORT_TARPIT;
 }
     break;
 case 78:
-#line 752 "parse.y"
+#line 730 "parse.y"
 {
 		pf_osfp_t fp;
 		yyvsp[0].string[strlen(yyvsp[0].string) - 1] = '\0';
@@ -1862,7 +1865,7 @@ case 78:
 	}
     break;
 case 79:
-#line 765 "parse.y"
+#line 743 "parse.y"
 {
 		if ((yyval.condition.match_arg = malloc(sizeof(struct addr))) == NULL)
 			yyerror("Out of memory");
@@ -1872,7 +1875,7 @@ case 79:
 	}
     break;
 case 80:
-#line 773 "parse.y"
+#line 751 "parse.y"
 {
 		if ((yyval.condition.match_arg = malloc(sizeof(struct addr))) == NULL)
 			yyerror("Out of memory");
@@ -1882,7 +1885,7 @@ case 80:
 	}
     break;
 case 81:
-#line 781 "parse.y"
+#line 759 "parse.y"
 {
 		if ((yyval.condition.match_arg = malloc(sizeof(struct condition_time))) == NULL)
 			yyerror("Out of memory");
@@ -1892,14 +1895,14 @@ case 81:
 	}
     break;
 case 82:
-#line 791 "parse.y"
+#line 769 "parse.y"
 {
 		yyval.timecondition.tm_start = yyvsp[-2].time;
 		yyval.timecondition.tm_end = yyvsp[0].time;
 	}
     break;
 case 83:
-#line 798 "parse.y"
+#line 776 "parse.y"
 {
 		int ispm = -1;
 		int hour, minute;
@@ -2155,7 +2158,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 820 "parse.y"
+#line 798 "parse.y"
 
 
 int
