@@ -1,31 +1,33 @@
 /*
- * Copyright 2002 Niels Provos <provos@citi.umich.edu>
+ * Copyright (c) 2002, 2003, 2004 Niels Provos <provos@citi.umich.edu>
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Niels Provos.
- * 4. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include <sys/param.h>
@@ -86,11 +88,11 @@ personality_init(void)
 }
 
 struct personality *
-personality_new(char *name)
+personality_new(const char *name)
 {
 	struct personality *pers, tmp;
 
-	tmp.name = name;
+	tmp.name = (char *)name;
 	if (SPLAY_FIND(perstree, &personalities, &tmp))
 		return (NULL);
 
@@ -113,7 +115,7 @@ personality_new(char *name)
 }
 
 struct personality *
-personality_clone(struct personality *person)
+personality_clone(const struct personality *person)
 {
 	struct personality *newperson;
 
@@ -141,12 +143,6 @@ personality_free(struct personality *pers)
 	SPLAY_REMOVE(perstree, &personalities, pers);
 
 	free(pers->name);
-
-	if (pers->xp_fprint) {
-		free(pers->xp_fprint->os_id);
-		free(pers->xp_fprint);
-	}
-
 	free(pers);
 }
 
@@ -170,10 +166,10 @@ personality_random(void)
 }
 
 struct personality *
-personality_find(char *name)
+personality_find(const char *name)
 {
 	struct personality tmp;
-	tmp.name = name;
+	tmp.name = (char *)name;
 	return (SPLAY_FIND(perstree, &personalities, &tmp));
 }
 
@@ -208,7 +204,7 @@ ip_personality(struct template *tmpl, uint16_t *pid)
 }
 
 struct personate *
-tcp_personality_test(struct tcp_con *con, struct personality *person,
+tcp_personality_test(const struct tcp_con *con, struct personality *person,
     uint8_t sndflags)
 {
 	uint8_t flags;
@@ -280,6 +276,12 @@ tcp_personality_test(struct tcp_con *con, struct personality *person,
 			return (NULL);
 		}
 	} else if ((flags & TH_FIN) && (flags & (TH_SYN|TH_ACK)) == 0) {
+		/*
+		 * If we get a FIN flag and do not allow fin scanning, then
+		 * we just let the regular state engine run its course.
+		 * Otherwise, we silently drop the packet, which indicates
+		 * that FIN scanning is allowed for TCP_STATE_LISTEN.
+		 */
 		if (person->disallow_finscan || con->state != TCP_STATE_LISTEN)
 			return (NULL);
 		return (&person_drop);
@@ -349,7 +351,7 @@ tcp_personality_seqinit(struct personality *person)
  */
 
 static uint32_t
-get_next_isn(struct template *tmpl, struct personality *person)
+get_next_isn(struct template *tmpl, const struct personality *person)
 {
 	uint32_t prev_isn = tmpl->seq;
 	uint32_t num_isns = tmpl->seqcalls;
@@ -358,14 +360,14 @@ get_next_isn(struct template *tmpl, struct personality *person)
         double amax = person->seqindex_amax;
         double a;
 
-	if ((amax - amin) >= 2) {
+	if ((amax - amin) >= 2.0) {
 		/* Select an 'a' halfway between acceptable limits.  The
 		 * value of 'a' should always be the same, not random.
 		 */
-		a = ((amax - amin) / 2) + amin;
+		a = ((amax - amin) / 2.0) + amin;
 	} else {
 		/* To get a valid number in a small range */
-                a = (uint32_t)ceil(((amax - amin) / 2) + amin);
+                a = (uint32_t)ceil(((amax - amin) / 2.0) + amin);
 	}
 	/* Constant ISN difference */
         if ((amax - amin) < 1 || gcd > (5 / 2 * person->seqindex_aconst))
@@ -395,7 +397,7 @@ tcp_personality_seq(struct template *tmpl, struct personality *person)
 		if (tmpl->timestamp == 0)
 			tmpl->timestamp = rand_uint32(honeyd_rand) % 1728000;
 		if (tmpl->seq == 0) {
-			if (person->seqt == SEQ_CONSTANT && person->val != 0)
+			if (person->seqt == SEQ_CONSTANT && person->valset)
 				tmpl->seq = person->val;
 			else
 				tmpl->seq = rand_uint32(honeyd_rand);
@@ -414,7 +416,8 @@ tcp_personality_seq(struct template *tmpl, struct personality *person)
 		if (tstamphz == -1)
 			tstamphz = 2;
 
-		/* Adjust so that the remaining subsecond ticks get
+		/* 
+		 * Adjust so that the remaining subsecond ticks get
 		 * counted next time.
 		 */
 		ticks = 1000000L / tstamphz;
@@ -423,16 +426,18 @@ tcp_personality_seq(struct template *tmpl, struct personality *person)
 
 		tmpl->timestamp += slowhz;
 	} else {
-		/* This is not the default.  Some stacks don't have
+		/*
+		 * This is not the default.  Some stacks don't have
 		 * any notion of time.
 		 */
 		slowhz = 0;
 		tmpl->timestamp = 0;
 	}
 
-	/* This is where new ISNs are generated.
-	 * The latest ISN is stored in tmpl->seq.
-	 * tmpl->seqcalls is the number of ISNs generated so far.
+	/* 
+	 * This is where new ISNs are generated.  The latest ISN is
+	 * stored in tmpl->seq.  tmpl->seqcalls is the number of ISNs
+	 * generated so far.
 	 */
 	switch(person->seqt) {
 	case SEQ_CONSTANT:
@@ -440,7 +445,7 @@ tcp_personality_seq(struct template *tmpl, struct personality *person)
 	case SEQ_TRIVIALTIME:
 	case SEQ_RI:
 		/* No time component.  May be required for high latency */
-		if (slowhz > 2) {
+		if (tmp.tv_sec > 2) {
 			uint32_t adjust;
 
 			adjust = rand_uint32(honeyd_rand) % 2048;
@@ -464,34 +469,61 @@ tcp_personality_seq(struct template *tmpl, struct personality *person)
 /* Default TCP options is timestamp, noop, noop */
 static char *default_opts = "tnn";
 
-char *
+int
+tcp_personality_match(struct tcp_con *con, int flags)
+{
+	struct template *tmpl = con->tmpl;
+	struct personality *person;
+
+	/* Find template and corresponding personality */
+	if (tmpl == NULL)
+		return (0);
+	person = tmpl->person;
+	if (person == NULL)
+		return (0);
+
+	return (tcp_personality_test(con, person, flags) != NULL);
+}
+
+int
 tcp_personality(struct tcp_con *con, uint8_t *pflags, int *pwindow, int *pdf,
-    uint16_t *pid)
+    uint16_t *pid, char **poptions)
 {
 	struct template *tmpl = con->tmpl;
 	struct personality *person;
 	struct personate *pers;
+	uint8_t flags = *pflags;
 
+	if (poptions != NULL)
+		*poptions = NULL;
+
+	/* XXX - We need to find some template to use here */
+
+	/* Find template and corresponding personality */
 	if (tmpl == NULL)
-		return (NULL);
+		return (-1);
 	person = tmpl->person;
 	if (person == NULL)
-		return (NULL);
+		return (-1);
 
-	if ((pers = tcp_personality_test(con, person, *pflags)) == NULL) {
+	if ((pers = tcp_personality_test(con, person, flags)) == NULL) {
 		/* Not a test case - but we still want to pretend */
 		ip_personality(tmpl, pid);
-		if (con->snd_una == 0 && (con->rcv_flags & TH_SYN))
+
+		/* Set the sequence number only on SYN segments */
+		if (con->snd_una == 0 && (flags & TH_SYN))
 			con->snd_una = tcp_personality_seq(tmpl, person);
 		/* If we support timestamps, always set them */
-		if (person->tstamphz >= 0)
-			return (default_opts);
-		return (NULL);
+		if (person->tstamphz >= 0 && poptions != NULL)
+			*poptions = default_opts;
+		return (-1);
 	}
 
 	*pwindow = pers->window;
 	*pflags = pers->flags;
 	*pdf = pers->df;
+	if (poptions != NULL)
+		*poptions = pers->options;
 
 	switch (pers->forceack) {
 	case ACK_ZERO:
@@ -509,7 +541,7 @@ tcp_personality(struct tcp_con *con, uint8_t *pflags, int *pwindow, int *pdf,
 	if (con->snd_una == 0)
 		con->snd_una = tcp_personality_seq(tmpl, person);
 
-	return (pers->options);
+	return (0);
 }
 
 #define SET(y, x, w, l) do { \
@@ -812,15 +844,17 @@ parse_tseq_si(struct personality *pers, char *s, char *end)
 	if (pers->seqt == SEQ_RI && minval <= SEQ_TRIVIALTIME_MAX)
 		minval = SEQ_TRIVIALTIME_MAX + 1;
 
-	/* With the mods above, this is now possible,
-	 * even though it is not supposed to happen.  This
-	 * means the fingerprints file is wrong.
+	/*
+	 * With the mods above, this is now possible, even though it
+	 * is not supposed to happen.  This means the fingerprints
+	 * file is wrong.
 	 */
 	if (minval > maxval) {
 		fprintf(stderr, "Warning: "
 		    "Impossible SI range in Class fingerprint \"%s\"\n",
 		    pers->name);
 	}
+
 	pers->seqindex_min = minval;
 	pers->seqindex_max = maxval;
 
@@ -878,6 +912,7 @@ parse_tseq(struct personality *pers, int off, char *line)
 
 			p2 += 4;
 			pers->val = strtol(p2, &e, 16);	
+			pers->valset = 1;
 		}
 		/* Improve IPID sequencing capability */
 		else if (strncasecmp(p2, "IPID=", 5) == 0) {
@@ -1157,7 +1192,8 @@ personality_line(struct personality *pers, char *line)
 	char *p, *p2;
 
 	/* Ignore additional nmap output */
-	if (strncasecmp(line, "SInfo", 5) == 0)
+	if (strncasecmp(line, "SInfo", 5) == 0 ||
+	    strncasecmp(line, "Class", 5) == 0)
 		return (0);
 
 	p2 = line;
@@ -1175,6 +1211,28 @@ personality_line(struct personality *pers, char *line)
 	return (-1);
 }
 
+/* Creates a new personality and details on how to deal with duplicates */
+
+struct personality *
+personality_config_new(const char *name, int lineno)
+{
+	struct personality *pers;
+	extern int honeyd_debug;
+
+	if ((pers = personality_new(name)) != NULL)
+		return (pers);
+
+	if (honeyd_debug > 1)
+		fprintf(stderr, "%d: Overwriting old fingerprint \"%s\"\n",
+		    lineno, name);
+
+	/* Find the old personality, and remove it */
+	pers = personality_find(name);
+	personality_free(pers);
+
+	return (personality_new(name));
+}
+
 /* Parsing Functions */
 
 int
@@ -1210,12 +1268,9 @@ personality_parse(FILE *fin)
 			for (p2 = p + strlen(p) - 1; isblank(*p2); p2--)
 				*p2 = '\0';
 
-			if ((pers = personality_new(p)) == NULL) {
-				fprintf(stderr,
-				    "%d: Ignoring duplicate fingerprint"
-				    " \"%s\"\n", lineno, p);
+			if ((pers = personality_config_new(p, lineno)) == NULL)
 				ignore = 1;
-			} else
+			else
 				ignore = 0;
 			continue;
 		}
